from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .BoundaryConditionState import BoundaryConditionState


@abaqus_class_doc
class VelocityBaseMotionBCState(BoundaryConditionState):
    """The VelocityBaseMotionBCState object stores the propagating data for a velocity base motion boundary
    condition in a step. One instance of this object is created internally by the VelocityBaseMotionBC object
    for each step. The instance is also deleted internally by the VelocityBaseMotionBC object. The
    VelocityBaseMotionBCState object has no constructor or methods. The VelocityBaseMotionBCState object is
    derived from the BoundaryConditionState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].boundaryConditionStates[name]

        The corresponding analysis keywords are:

        - BASE MOTION
    """

    #: A SymbolicConstant specifying the propagation state of the amplitude reference. Possible
    #: values are UNSET, SET, UNCHANGED, FREED, and MODIFIED.
    amplitudeState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the BoundaryConditionState object. Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - PROPAGATED_FROM_BASE_STATE
    #: - MODIFIED_FROM_BASE_STATE
    #: - DEACTIVATED_FROM_BASE_STATE
    #: - BUILT_INTO_MODES
    status: Optional[SymbolicConstant] = None

    #: A String specifying the name of the amplitude reference. The String is empty if the
    #: boundary condition has no amplitude reference.
    amplitude: str = ""



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .InteractionState import InteractionState


@abaqus_class_doc
class SurfaceToSurfaceExpState(InteractionState):
    """The SurfaceToSurfaceExpState object stores the propagating data for a SurfaceToSurfaceContactExp object.
    One instance of this object is created internally by the SurfaceToSurfaceContactExp object for each step.
    The instance is also deleted internally by the SurfaceToSurfaceContactExp object. The
    SurfaceToSurfaceExpState object has no constructor or methods. The SurfaceToSurfaceExpState object is
    derived from the InteractionState object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]

        The corresponding analysis keywords are:

        - CONTACT CONTROLS
        - CONTACT PAIR
    """

    #: A SymbolicConstant specifying the propagation state of the **interactionProperty** member.
    #: Possible values are UNSET, SET, UNCHANGED, and FREED.
    interactionPropertyState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **contactControls** member.
    #: Possible values are UNSET, SET, UNCHANGED, and FREED.
    contactControlsState: Optional[SymbolicConstant] = None

    #: A String specifying the name of the ContactProperty object associated with this
    #: interaction.
    interactionProperty: str = ""

    #: A String specifying the name of the ContactControl object associated with this
    #: interaction.
    contactControls: str = ""

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None



from typing import Optional, Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Connector.ConnectorSection import ConnectorSection
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .TransverseShearBeam import TransverseShearBeam
from .TransverseShearShell import TransverseShearShell


@abaqus_class_doc
class Section(ConnectorSection):
    @abaqus_method_doc
    def TransverseShearBeam(
        self,
        scfDefinition: Literal[C.VALUE, C.ANALYSIS_DEFAULT, C.COMPUTED],
        k23: Optional[float] = None,
        k13: Optional[float] = None,
        slendernessCompensation: Union[Literal[C.COMPUTED], float] = 0,
    ) -> TransverseShearBeam:
        """This method creates a TransverseShearBeam object.

        .. note::
            This function can be accessed by::

                mdb.models[name].sections[name].TransverseShearBeam
                session.odbs[name].sections[name].TransverseShearBeam

        Parameters
        ----------
        scfDefinition
            A SymbolicConstant specifying how slenderness compensation factor of the section is
            given. Possible values are ANALYSIS_DEFAULT, COMPUTED, and VALUE.
        k23
            None or a Float specifying the k23 shear stiffness of the section. The default value is
            None.
        k13
            None or a Float specifying the k13 shear stiffness of the section. The default value is
            None.
        slendernessCompensation
            The SymbolicConstant COMPUTED or a Float specifying the slenderness compensation factor
            of the section. The default value is 0.25.

        Returns
        -------
        TransverseShearBeam
            A TransverseShearBeam object.
        """
        self.beamTransverseShear = transverseShearBeam = TransverseShearBeam(
            scfDefinition, k23, k13, slendernessCompensation
        )
        return transverseShearBeam

    @abaqus_method_doc
    def TransverseShearShell(self, k11: float, k22: float, k12: float) -> TransverseShearShell:
        """This method creates a TransverseShearShell object.

        .. note::
            This function can be accessed by::

                mdb.models[name].sections[name].TransverseShearShell
                session.odbs[name].sections[name].TransverseShearShell

        Parameters
        ----------
        k11
            A Float specifying the shear stiffness of the section in the first direction.
        k22
            A Float specifying the shear stiffness of the section in the second direction.
        k12
            A Float specifying the coupling term in the shear stiffness of the section.

        Returns
        -------
        TransverseShearShell
            A TransverseShearShell object.
        """
        self.transverseShear = transverseShearShell = TransverseShearShell(k11, k22, k12)
        return transverseShearShell



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .InteractionState import InteractionState


@abaqus_class_doc
class RadiationToAmbientState(InteractionState):
    """The RadiationToAmbientState object stores the propagating data for a RadiationToAmbient object. One
    instance of this object is created internally by the RadiationToAmbient object for each step. The instance
    is also deleted internally by the RadiationToAmbient object. The RadiationToAmbientState object has no
    constructor or methods. The RadiationToAmbientState object is derived from the InteractionState object.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].steps[name].interactionStates[name]

        The corresponding analysis keywords are:

        - SRADIATE
    """

    #: A Float specifying the ambient temperature.
    ambientTemperature: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the **ambientTemperature** member.
    #: Possible values are UNSET, SET, UNCHANGED, and FREED.
    ambientTemperatureState: Optional[SymbolicConstant] = None

    #: A SymbolicConstant specifying the propagation state of the **ambientTemperatureAmp**
    #: member. Possible values are UNSET, SET, UNCHANGED, and FREED.
    ambientTemperatureAmpState: Optional[SymbolicConstant] = None

    #: A Float specifying the emissivity.
    emissivity: Optional[float] = None

    #: A SymbolicConstant specifying the propagation state of the **emissivity** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    emissivityState: Optional[SymbolicConstant] = None

    #: A String specifying the name of the Amplitude object that gives the variation of the
    #: ambient temperature with time.
    ambientTemperatureAmp: str = ""

    #: A SymbolicConstant specifying the propagation state of the InteractionState object.
    #: Possible values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: Optional[SymbolicConstant] = None



from typing import Optional

from abqpy.decorators import abaqus_class_doc

from .OdbSet import OdbSet


@abaqus_class_doc
class OdbPretensionSection:
    """The pretension section object is used to define an assembly load. It associates a pretension node with a
    pretension section.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name].rootAssembly.pretensionSections[i]
    """

    #: An OdbSet object specifying the node set containing the pretension node.
    node: OdbSet = OdbSet("set", ())

    #: An OdbSet object specifying the element set that defines the pretension section.
    element: OdbSet = OdbSet("set", ())

    #: An OdbSet object specifying the surface set that defines the pretension section.
    surface: OdbSet = OdbSet("set", ())

    #: A tuple of Floats specifying the components of the normal to the pretension section.
    normal: Optional[float] = None



from typing import Optional, Tuple

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import OFF, Boolean


@abaqus_class_doc
class KeywordBlock:
    """The KeywordBlock object contains a representation of its model in the Abaqus input file format. You may
    edit the contents of the KeywordBlock to add solver functionality that is not supported by Abaqus/CAE. As a
    general rule, edits to the KeywordBlock object should be made as the last step prior to writing the actual
    Abaqus input file, thus avoiding possible conflicts with changes made using other MDB commands. The
    KeywordBlock object has no constructor. A KeywordBlock object is created when you create a model object. A
    model object contains only one KeywordBlock object.

    .. note::
        This object can be accessed by::

            mdb.models[name].keywordBlock
    """

    #: A Boolean specifying whether the Keywords Editor has been used to change the model.
    edited: Boolean = OFF

    #: A Float specifying the value of the counter associated with the Mdb object at the most
    #: recent synchronization.
    lastSynchCount: Optional[float] = None

    #: A tuple of Strings specifying a sequence of Strings that is identical to the information
    #: written to the Abaqus input file. Each String in the sequence represents an Abaqus input
    #: file keyword along with the parameters and data lines associated with the keyword. A
    #: String can also be a comment in the input file. You initialize this data member by
    #: calling synchVersions. After you initialize the data member, you use calls to replace
    #: and insert to record your edits in the correct location. If the last call to
    #: synchVersions used the argument **storeNodesAndElements** = False, the entry for the
    #: keywords NODE and ELEMENT will contain only the keyword and its parameters, not the data
    #: lines.
    sieBlocks: Tuple[str, ...] = ()

    @abaqus_method_doc
    def setValues(self, edited: Boolean = OFF):
        """This method modifies the KeywordBlock object.

        Parameters
        ----------
        edited
            A Boolean specifying whether this objects **sieBlocks** member has been edited. Setting
            edited=False will set the **sieBlocks** member to an empty tuple, thereby discarding all
            previous edits.
        """
        ...

    @abaqus_method_doc
    def insert(self, position: int, text: str):
        """This method inserts a String at a specified position in the **sieBlocks** member.

        Parameters
        ----------
        position
            An Int specifying the position in the **sieBlocks** member after which the new string will
            be inserted.
        text
            A String specifying the text to be inserted. The text represents an Abaqus input file
            keyword and its associated data

        Raises
        ------
        IndexError
        """
        ...

    @abaqus_method_doc
    def replace(self, position: int, text: str):
        """This method replaces a String at a specified position in the **sieBlocks** member.

        Parameters
        ----------
        position
            An Int specifying the position of the String to be replaced in the **sieBlocks** member.
        text
            A String specifying the text to be replaced. The text represents an Abaqus input file
            keyword and its associated data.

        Raises
        ------
        IndexError
        """
        ...

    @abaqus_method_doc
    def synchVersions(self, storeNodesAndElements: Boolean):
        """This method synchronizes, or merges, the edits made in this object with those made in the model using
        other scripting commands or the user interface. The synchVersions method updates the **sieBlocks**
        member. The **sieBlocks** member is empty prior to the first call to synchVersions. As a side effect,
        synchVersions sets **lastSynchCount** to the current value of the counter associated with the Mdb
        object, which is used to determine if synchronization is necessary.

        Parameters
        ----------
        storeNodesAndElements
            A Boolean specifying whether the nodal coordinates and element connectivities (i.e. the
            data lines for the *NODE and *ELEMENT keyword blocks) are to be stored in the
            **sieBlocks** member. All other keywords and their data lines are always stored. The
            default value is True. If **storeNodesAndElements** is True, the size of the keywordBlock
            data will be similar to that of the input file. Since the KeywordBlock is stored in the
            Abaqus/CAE database, this will result in a larger database. It will also result in a
            slower execution of the synchVersions command. If **storeNodesAndElements** is False, the
            data lines are not stored in **sieBlocks**. Consequently, only set
            **storeNodesAndElements** = True if you wish to make changes to the **NODE** or **ELEMENT** data
            lines themselves. If your task is limited to reading nodal coordinates and element
            connectivities (i.e. not editing this information) then it is generally better to access
            this information from other parts of the Mdb.

        Raises
        ------
        IndexError
        """
        ...



from typing import Optional

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import MAXIMUM, ON, Boolean, SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .GeometricRestriction import GeometricRestriction


@abaqus_class_doc
class DrillControl(GeometricRestriction):
    """The DrillControl object defines a drill control geometric restriction. The DrillControl object is derived
    from the GeometricRestriction object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]
    """

    #: A String specifying the geometric restriction repository key.
    name: str

    #: A VertexArray object of length 2 specifying the direction of the drill axis positioned
    #: at the **csys** origin. Instead of through a ConstrainedSketchVertex, each point may be specified through a
    #: tuple of coordinates.
    clientDirection: tuple

    #: A Region object specifying the region to which the geometric restriction is applied.
    #: When used with a TopologyTask, there is no default value. When used with a ShapeTask,
    #: the default value is MODEL.
    region: Region

    #: None or a DatumCsys object specifying the local coordinate system. If **csys** = None, the
    #: global coordinate system is used. When this member is queried, it returns an Int. The
    #: default value is None.
    csys: Optional[int] = None

    #: A Float specifying the draw angle. The default value is 0.0.
    drawAngle: float = 0

    #: None or a Region object specifying the master point used when **masterPointDetermination** is
    #: SPECIFY. The default value is None.
    masterPoint: Optional[str] = None

    #: A SymbolicConstant specifying the rule for assigning point priority. Possible values are
    #: MAXIMUM, MINIMUM, and SPECIFY. The default value is MAXIMUM.
    masterPointDetermination: SymbolicConstant = MAXIMUM

    #: A Boolean specifying whether to ignore the geometric restriction in the first design
    #: cycle. The default value is ON.
    presumeFeasibleRegionAtStart: Boolean = ON

    #: A Float specifying the geometric tolerance in the 1-direction. The default value is
    #: 0.01.
    tolerance1: float = 0

    #: A Float specifying the geometric tolerance in the 2-direction. The default value is
    #: 0.01.
    tolerance2: float = 0

    #: A Float specifying the geometric tolerance in the 3-direction. The default value is
    #: 0.01.
    tolerance3: float = 0

    #: A Float specifying the undercut tolerance. The default value is 0.0.
    undercutTolerance: float = 0

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        clientDirection: tuple,
        region: Region,
        csys: Optional[int] = None,
        drawAngle: float = 0,
        masterPoint: Optional[str] = None,
        masterPointDetermination: Literal[C.SPECIFY, C.MINIMUM, C.MAXIMUM] = MAXIMUM,
        presumeFeasibleRegionAtStart: Boolean = ON,
        tolerance1: float = 0,
        tolerance2: float = 0,
        tolerance3: float = 0,
        undercutTolerance: float = 0,
    ):
        """This method creates a DrillControl object.

        .. note::
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].DrillControl

        Parameters
        ----------
        name
            A String specifying the geometric restriction repository key.
        clientDirection
            A VertexArray object of length 2 specifying the direction of the drill axis positioned
            at the **csys** origin. Instead of through a ConstrainedSketchVertex, each point may be specified through a
            tuple of coordinates.
        region
            A Region object specifying the region to which the geometric restriction is applied.
            When used with a TopologyTask, there is no default value. When used with a ShapeTask,
            the default value is MODEL.
        csys
            None or a DatumCsys object specifying the local coordinate system. If **csys** = None, the
            global coordinate system is used. When this member is queried, it returns an Int. The
            default value is None.
        drawAngle
            A Float specifying the draw angle. The default value is 0.0.
        masterPoint
            None or a Region object specifying the master point used when **masterPointDetermination** is
            SPECIFY. The default value is None.
        masterPointDetermination
            A SymbolicConstant specifying the rule for assigning point priority. Possible values are
            MAXIMUM, MINIMUM, and SPECIFY. The default value is MAXIMUM.
        presumeFeasibleRegionAtStart
            A Boolean specifying whether to ignore the geometric restriction in the first design
            cycle. The default value is ON.
        tolerance1
            A Float specifying the geometric tolerance in the 1-direction. The default value is
            0.01.
        tolerance2
            A Float specifying the geometric tolerance in the 2-direction. The default value is
            0.01.
        tolerance3
            A Float specifying the geometric tolerance in the 3-direction. The default value is
            0.01.
        undercutTolerance
            A Float specifying the undercut tolerance. The default value is 0.0.

        Returns
        -------
        DrillControl
            A DrillControl object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        csys: Optional[int] = None,
        drawAngle: float = 0,
        masterPoint: Optional[str] = None,
        masterPointDetermination: Literal[C.SPECIFY, C.MINIMUM, C.MAXIMUM] = MAXIMUM,
        presumeFeasibleRegionAtStart: Boolean = ON,
        tolerance1: float = 0,
        tolerance2: float = 0,
        tolerance3: float = 0,
        undercutTolerance: float = 0,
    ):
        """This method modifies the DrillControl object.

        Parameters
        ----------
        csys
            None or a DatumCsys object specifying the local coordinate system. If **csys** = None, the
            global coordinate system is used. When this member is queried, it returns an Int. The
            default value is None.
        drawAngle
            A Float specifying the draw angle. The default value is 0.0.
        masterPoint
            None or a Region object specifying the master point used when **masterPointDetermination** is
            SPECIFY. The default value is None.
        masterPointDetermination
            A SymbolicConstant specifying the rule for assigning point priority. Possible values are
            MAXIMUM, MINIMUM, and SPECIFY. The default value is MAXIMUM.
        presumeFeasibleRegionAtStart
            A Boolean specifying whether to ignore the geometric restriction in the first design
            cycle. The default value is ON.
        tolerance1
            A Float specifying the geometric tolerance in the 1-direction. The default value is
            0.01.
        tolerance2
            A Float specifying the geometric tolerance in the 2-direction. The default value is
            0.01.
        tolerance3
            A Float specifying the geometric tolerance in the 3-direction. The default value is
            0.01.
        undercutTolerance
            A Float specifying the undercut tolerance. The default value is 0.0.
        """
        ...



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from .CommandRegister import CommandRegister


@abaqus_class_doc
class RegisteredDictionary(CommandRegister, dict):
    """This class allows you to create a dictionary that can be queried from the GUI and is capable of notifying
    the GUI when the contents of the dictionary change. The keys to a RegisteredDictionary must be either
    strings or integers. The RegisteredDictionary object is derived from the CommandRegister object.

    .. note::
        This object can be accessed by::

            import customKernel
    """

    @abaqus_method_doc
    def __init__(self):
        """This method creates a RegisteredDictionary object.

        .. note::
            This function can be accessed by::

                customKernel.RegisteredDictionary

        Returns
        -------
        RegisteredDictionary
            A RegisteredDictionary object.
        """
        super().__init__()

    @abaqus_method_doc
    def Methods(self):
        """The RegisteredDictionary object supports the same methods as a Python dictionary.

        In addition, the RegisteredDictionary object supports the changeKey method.
        """
        # TODO: Implement this method
        ...

    @abaqus_method_doc
    def changeKey(self, fromName: str, toName: str):
        """This method changes the name of a key in the dictionary.

        Parameters
        ----------
        fromName
            A String or an integer specifying the name of the key to be changed.
        toName
            A String or an integer specifying the new name for the key.
        """
        if fromName in self:
            self[toName] = self[fromName]
            del self[fromName]
